# Python script to produce a comparison matrix of the geodesic distances
# between all trees in one file with all trees in a second file that can be input into matlab.
# Input:  two files with one tree per line in Newick format

# Note:  must be able to call java from the command line in the directory
# containing this script by typing 'java'

import argparse
import os
import re

# parameters
dist_file_name_default = 'raw_GTP_output.txt'    # holds the output of GTP
comp_file_name_default = 'comp_matrix.txt'   # file holding the comparison matrix for input for matlab

# Read in arguments:  file name
parser = argparse.ArgumentParser(description ='Creates distance matrix of the distances from the trees in the first file (rows) to the trees in the second file (columns).')
parser.add_argument('tree_file_1_name', help='name file containing trees in Newick format')
parser.add_argument('tree_file_2_name', help='name file containing trees in Newick format')
parser.add_argument('-u', action='store_true', dest = 'unrooted', help='Use for unrooted trees.  Default is rooted.')
parser.add_argument('-d',action='store', default = dist_file_name_default, dest = 'dist_file_name',help = 'optional raw distance file name. Default is ' + dist_file_name_default)
parser.add_argument('-o',action='store', default = comp_file_name_default, dest = 'comp_file_name',help = 'optional comparison matrix file name. Default is ' + comp_file_name_default)
parser.add_argument('-a', action = 'store', default = 'analysis.jar', dest = 'analysis_version',help = 'optional analysis version.  Default is analysis.jar')

args = parser.parse_args()
	
num_trees_1 = sum(1 for line in open(args.tree_file_1_name))
num_trees_2 = sum(1 for line in open(args.tree_file_2_name))

# Call GTP on the input tree file.

if os.path.exists(args.dist_file_name):
	print(args.dist_file_name + ' exists - skipping generating it')
	
else:
	
	if args.unrooted == True:
		os.system('java -jar ' + args.analysis_version + ' -a gtp_twofiles -u -o ' + args.dist_file_name + ' -f ' + args.tree_file_2_name + ' ' + args.tree_file_1_name)

	else:
		os.system('java -jar ' + args.analysis_version + ' -a gtp_twofiles -o ' + args.dist_file_name + ' -f ' + args.tree_file_2_name + ' ' + args.tree_file_1_name)

#  GTP outputs file with one line per tree pair, with the first column holding the index of the starting tree,
# the second column holding the index of the target tree, and the third column holding the geodesic distance 
# between them.  The trees are indexed in the order they appear in the two input files, starting at 0 

# Convert this output to a file containing the dissimilarity matrix (with columns separated by spaces).
if os.path.exists(args.comp_file_name):
	print(args.comp_file_name + ' exists - skipping generating the dissimilarity matrix file')
	
else:	
	
	geo_dist_file = open(args.dist_file_name,'r')

	comp_matrix = [ [0]*num_trees_2 for x in xrange(num_trees_1)]

	for line in geo_dist_file:
		match_line = re.match(r'([0-9]+)\s([0-9]+)\s([0-9\.]+)',line)
	
		if (match_line): 
			comp_matrix[int(match_line.group(1))][int(match_line.group(2))]	= match_line.group(3)

	geo_dist_file.close()

	# print the geodesic distances as a dissimilarity matrix to the output file.
	o = open(args.comp_file_name,'w')
	
	for row in range(num_trees_1):
		for col in range(num_trees_2):
			o.write(str(comp_matrix[row][col]) + ' ')
		o.write('\n')
		
	o.close()

# Copyright (C) 2013 Aasa Feragen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Author: Megan Owen <megan dot owen at uwaterloo dot ca>
#



