% hypothesis_test_setmeans performs permutation tests on mean and
% variance of two populations of trees using set means
%
% Defined in the paper 'Tree-space statistics and approximations for 
% large-scale analysis of anatomical trees (Aasa Feragen, Megan Owen, Jens 
% Petersen, Mathilde M. W. Wille, Laura H. Thomsen, Asger Dirksen, and 
% Marleen de Bruijne), Information Processing in Medical Imaging 2013, 
% Lecture Notes in Computer Science 7917'.
%
% [h, p] = hypothesis_test_setmeans(D_set, S_indices, classification_vector, test_type, permutation_nr)
%
% Assume given 
%  - a dataset X with m elements in data space Y, such that X spans Y 
%    as well as possible 
%  - a partitioned subset S = S1 union S2 with n1 and n2 elements, 
%    respectively, with #S1 = n1, #S2 = n2, #S = n = n1 + n2
%  - an (m x m) distancematrix D_set, where D(i,j) is the distance 
%    from x_i in X to x_j in X
%  - a binary classification vector
%
% We perform permutation tests on means and variance
%
% Input variables:
%  D_set                   
%  S_indices               the indices of the elements of the set S in the 
%                          set X
%  classification_vector   classifies S into S1 and S2
%  test_type               'T' (test on set means, default)
%                          'S1' (test on variance)
%                          'S2' (test on scatter)
%  permutation_nr          number of permutations used
%
% Output variables:
%  h    Takes values 1 (reject 0-hyp) and 0 (do not reject) at significance 
%       level 0.05
%  p    p-value for rejecting 0-hyp
%
% Copyright (C) 2013 Aasa Feragen
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%
% Author: Aasa Feragen <aasa at diku dot dk>
% 
% 2013-06-16 Aasa Feragen <aasa at diku dot dk>
% * Initial revision

function [h, p] = hypothesis_test_setmeans(D_set, S_indices, classification_vector, test_type, permutation_nr)    
    
    % Compute test statistic on given class partition:
        
    orig_t = t_stat(D_set, S_indices, classification_vector, test_type);

    %----------------------------------------------------------------------
    
    % Perform permutation test

    for i = 1 : permutation_nr
        disp(['Performing permutation ', num2str(i)]);
        random_classification_vector_indices = randperm(numel(classification_vector));
        permuted_classification_vector = classification_vector(random_classification_vector_indices);
        perm_t(i) = t_stat(D_set, S_indices, permuted_classification_vector, test_type);       
    end
    
    %----------------------------------------------------------------------
    
    % Compute p-value and reject or accept hypothesis

    p = (1/(permutation_nr + 1))*(1 + numel(find(perm_t >= orig_t)));
    if p < 0.05
        h = 1;
    else
        h = 0;
    end

end

function t = t_stat(D_set, S_indices, classification_vector, test_type)
    % Compute the t statistic
    
    if strcmp(test_type, 'T')
        % Test for equality of set means
                    
        % Compute an (m x n) distancematrix D_set_sample, where D(i,j) is the distance from x_i in X to s_j in S
        D_set_sample = D_set(:, S_indices);

        class1_indices = classification_vector == 1;
        class2_indices = classification_vector == 0;
        class1_distmatrix = D_set_sample(:, class1_indices);        
        class2_distmatrix = D_set_sample(:, class2_indices);
        class1_set_mean = set_mean(class1_distmatrix);
        class2_set_mean = set_mean(class2_distmatrix);
  
        t = D_set(class1_set_mean, class2_set_mean);

    elseif strcmp(test_type, 'S1')
        % Test of variance using the set mean       
    
        % Compute an (m x n) distancematrix D_set_sample, where D(i,j) is the distance from x_i in X to s_j in S
        D_set_sample = D_set(:, S_indices);
            
        class1_indices = classification_vector == 1;
        class2_indices = classification_vector == 0;
        class1_distmatrix = D_set_sample(:, class1_indices);
        class2_distmatrix = D_set_sample(:, class2_indices);
        class1_set_mean = set_mean(class1_distmatrix);
        class2_set_mean = set_mean(class2_distmatrix);
        class1_distances_to_mean = squeeze((class1_distmatrix(class1_set_mean,:)));
        class2_distances_to_mean = squeeze((class2_distmatrix(class2_set_mean,:)));
        class1_scatter = sum(class1_distances_to_mean.^2);
        class2_scatter = sum(class2_distances_to_mean.^2);

        class1_size = numel(class1_indices);
        class2_size = numel(class2_indices);
        t = norm(class1_scatter/class1_size - class2_scatter/class2_size);  
        
    elseif strcmp(test_type, 'S2')
        % using class spread
        
        % Compute an (m x n) distancematrix D_set_set, where D(i,j) is the distance from s_i in S to s_j in S
        D_set_set = D_set(S_indices, S_indices);
        
        class1_indices = find(classification_vector == 1);
        class2_indices = find(classification_vector == 0);
        class1_distmatrix = D_set_set(class1_indices, class1_indices);
        class2_distmatrix = D_set_set(class2_indices, class2_indices);  
        squared_class1_distmatrix = class1_distmatrix.^2;
        class1_spread = sum(squared_class1_distmatrix(:));
        squared_class2_distmatrix = class2_distmatrix.^2;
        class2_spread = sum(squared_class2_distmatrix(:));   
        t = norm(class1_spread - class2_spread);    
    end
end