% hypothesis_test_sturmmeans performs permutation tests on mean and
% variance of two populations of trees using Sturm means
%
% Defined in the paper 'Tree-space statistics and approximations for 
% large-scale analysis of anatomical trees (Aasa Feragen, Megan Owen, Jens 
% Petersen, Mathilde M. W. Wille, Laura H. Thomsen, Asger Dirksen, and 
% Marleen de Bruijne), Information Processing in Medical Imaging 2013, 
% Lecture Notes in Computer Science 7917'.
%
% [h, p] = hypothesis_test_sturmmeans(input_tree_txt_file, classification_vector, test_type, permutation_nr)
%
% Given a dataset Z = X union Y of trees printed in a single text file, along
% with a binary classification vector, we perform permutation tests on
% means and variance
%
% Input variables:
%  input_tree_txt_file      .txt file with all dataset trees in Newick 
%                           format, see the file 'airways_dataset.txt' for
%                           an example
%  classification_vector    classifies Z into X and Y 
%  test_type                'T' (default, tests equality of means)
%                           'S1' (tests equality of variance)
%  permutation_nr           number of permutations used
%
% Output variables:
%  h    Takes values 1 (reject 0-hyp) and 0 (do not reject) at significance 
%       level 0.05
%  p    p-value for rejecting 0-hyp
%
% Copyright (C) 2013 Aasa Feragen
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%
% Author: Aasa Feragen <aasa at diku dot dk>
% 
% 2013-06-16 Aasa Feragen <aasa at diku dot dk>
% * Initial revision

function [h, p] = hypothesis_test_sturmmeans(input_tree_txt_file, classification_vector, test_type, permutation_nr)
           
    system('rm output1.txt');
    system('rm output2.txt');
    
    % Compute test statistic on given class partition:

    orig_t = t_stat(input_tree_txt_file, classification_vector, test_type);
    

    %----------------------------------------------------------------------
    
    % Perform permutation test

    for i = 1 : permutation_nr
        disp(['Performing permutation ', num2str(i)]);
        random_classification_vector_indices = randperm(numel(classification_vector));
        permuted_classification_vector = classification_vector(random_classification_vector_indices);
        perm_t(i) = t_stat(input_tree_txt_file, permuted_classification_vector, test_type);        
    end
    
    %----------------------------------------------------------------------
    
    % Compute p-value and reject or accept hypothesis

    p = (1/(permutation_nr + 1))*(numel(find(perm_t >= orig_t)) + 1);
    if p < 0.05
        h = 1;
    else
        h = 0;
    end    
    
    %----------------------------------------------------------------------
    
    % Clean up.
    
    system('rm class1_trees.txt class2_trees.txt');
        
end

function t = t_stat(input_tree_txt_file, classification_vector, test_type)
    % Compute t statistic
    
    if strcmp(test_type, 'T')
        % Test for equality of Sturm means

        % Find indices of the two classes        
        class1_indices = find(classification_vector == 1);
        class2_indices = find(classification_vector == 0);
        
        % Read data trees
        fid = fopen(input_tree_txt_file);
        s = textscan(fid, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{1};
        fclose(fid);
        
        % Make txt files with class data trees
        fid1 = fopen('class1_trees.txt', 'w');
        for i = 1 : numel(class1_indices)
            fprintf(fid1, '%s\n', s{class1_indices(i)});
        end
        fclose(fid1);        
        
        fid2 = fopen('class2_trees.txt', 'w');
        for i = 1 : numel(class2_indices)
            fprintf(fid2, '%s\n', s{class2_indices(i)});
        end
        fclose(fid2);
        
        %------------------------------------------------------------------
        % Compute the Sturm means of the two sets
        
        display('computing first mean');
        system('java -jar SturmMean.jar -n 10000000 -e 0.01 -o out1.txt class1_trees.txt');
        
        % Fish the mean tree out of the file
        fid1 = fopen('out1.txt');
        s = textscan(fid1, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{end};
        s = s{end};
        fclose(fid1);
        fid1 = fopen('mean1.txt', 'w');
        fprintf(fid1, '%s', s);
        fclose(fid1);
        
        display('computing second mean');
        system('java -jar SturmMean.jar -n 10000000 -e 0.01 -o out2.txt class2_trees.txt');
        
        % Fish the mean tree out of the file        
        fid2 = fopen('out2.txt');
        s = textscan(fid2, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{end};
        s = s{end};
        fclose(fid2);        
        fid2 = fopen('mean2.txt', 'w');
        fprintf(fid2, '%s', s);
        fclose(fid2);

        %------------------------------------------------------------------
        % Compute the t statistic
        
        display('Computing t statistic');
        system('python create_comp_matrix.py mean1.txt mean2.txt');
        t = load('comp_matrix.txt');
        system('rm comp_matrix.txt');
        system('rm mean1.txt');
        system('rm mean2.txt');    
        system('rm class1_trees.txt');
        system('rm class2_trees.txt');
        system('rm out1.txt');
        system('rm out2.txt');
        system('rm raw_GTP_output.txt');

    elseif strcmp(test_type, 'S1')
        % Testing for difference in variance; variances defined using
        % Sturm mean
        
        % Find indices of the two classes        
        class1_indices = find(classification_vector == 1);
        class2_indices = find(classification_vector == 0);
        
        % Read data trees
        fid = fopen(input_tree_txt_file);
        s = textscan(fid, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{1};
        fclose(fid);
        
        % Make txt files with class data trees
        fid1 = fopen('class1_trees.txt', 'w');
        for i = 1 : numel(class1_indices)
            fprintf(fid1, '%s\n', s{class1_indices(i)});
        end
        fclose(fid1);        
        
        fid2 = fopen('class2_trees.txt', 'w');
        for i = 1 : numel(class2_indices)
            fprintf(fid2, '%s\n', s{class2_indices(i)});
        end
        fclose(fid2);
        
        %------------------------------------------------------------------
        % Compute the Sturm means of the two sets
        
        display('computing first mean');
        system('java -jar SturmMean.jar -n 10000000 -e 0.01 -o out1.txt class1_trees.txt');
        
        % Fish the mean tree out of the file
        fid1 = fopen('out1.txt');
        s = textscan(fid1, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{end};
        s = s{end};
        fclose(fid1);
        fid1 = fopen('mean1.txt', 'w');
        fprintf(fid1, '%s', s);
        fclose(fid1);
        
        display('computing second mean');
        system('java -jar SturmMean.jar -n 10000000 -e 0.01 -o out2.txt class2_trees.txt');
        
        % Fish the mean tree out of the file        
        fid2 = fopen('out2.txt');
        s = textscan(fid2, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{end};
        s = s{end};
        fclose(fid2);        
        fid2 = fopen('mean2.txt', 'w');
        fprintf(fid2, '%s', s);
        fclose(fid2);
        
        %-------------------------------------------------------------------
        
        % Compute the S1 statistic
        
        display('computing s1 statistic');
        system('python create_comp_matrix.py mean1.txt class1_trees.txt');
        res1 = load('comp_matrix.txt');
        system('python create_comp_matrix.py mean2.txt class2_trees.txt');
        res2 = load('comp_matrix.txt');
        class1_size = numel(class1_indices);
        class2_size = numel(class2_indices);
        t = norm(sum(res1.^2)/class1_size - sum(res2.^2)/class2_size);
        system('rm comp_matrix.txt');
        system('rm mean1.txt');
        system('rm mean2.txt');        
        system('rm out1.txt');
        system('rm out2.txt');
        system('rm raw_GTP_output.txt');
    end
end